@@ -19,7 +19,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 #include <stdio.h>
 #include <stdlib.h>
@@ -33,6 +33,7 @@
 
 #include "arguments.h"
 #include "utils.h"
+#include "version.h"
 
 /*---------------------------------------------------------------------------*/
 static int build_shortopts(const struct option *a_options,
@@ -128,6 +129,11 @@ static void print_usage(const struct option longopts[],
 	exit(0);
 }
 
+static void print_version() {
+	printf("%s\n", LXC_VERSION);
+	exit(0);
+}
+
 static void print_help(const struct lxc_arguments *args, int code)
 {
 	fprintf(stderr, "\
@@ -140,6 +146,7 @@ Common options :\n\
   -P, --lxcpath=PATH               Use specified container path\n\
   -?, --help                       Give this help list\n\
       --usage                      Give a short usage message\n\
+      --version                    Print the version number\n\
 \n\
 Mandatory or optional arguments to long options are also mandatory or optional\n\
 for any corresponding short options.\n\
@@ -147,16 +154,37 @@ for any corresponding short options.\n\
 See the %s man page for further information.\n\n",
 	args->progname, args->help, args->progname);
 
+	if (args->helpfn)
+		args->helpfn(args);
 	exit(code);
 }
 
+static int lxc_arguments_lxcpath_add(struct lxc_arguments *args,
+				     const char *lxcpath)
+{
+	if (args->lxcpath_additional != -1 &&
+	    args->lxcpath_cnt > args->lxcpath_additional) {
+		fprintf(stderr, "This command only accepts %d -P,--lxcpath arguments\n",
+			args->lxcpath_additional + 1);
+		exit(EXIT_FAILURE);
+	}
+
+	args->lxcpath = realloc(args->lxcpath, (args->lxcpath_cnt + 1) *
+				 sizeof(args->lxcpath[0]));
+	if (args->lxcpath == NULL) {
+		lxc_error(args, "no memory");
+		return -ENOMEM;
+	}
+	args->lxcpath[args->lxcpath_cnt++] = lxcpath;
+	return 0;
+}
+
 extern int lxc_arguments_parse(struct lxc_arguments *args,
 			       int argc, char * const argv[])
 {
 	char shortopts[256];
 	int  ret = 0;
 
-	args->lxcpath = default_lxc_path();
 	ret = build_shortopts(args->options, shortopts, sizeof(shortopts));
 	if (ret < 0) {
 		lxc_error(args, "build_shortopts() failed : %s",
@@ -174,10 +202,15 @@ extern int lxc_arguments_parse(struct lxc_arguments *args,
 		case 'n': 	args->name = optarg; break;
 		case 'o':	args->log_file = optarg; break;
 		case 'l':	args->log_priority = optarg; break;
-		case 'c':	args->console = optarg; break;
 		case 'q':	args->quiet = 1; break;
-		case 'P':	args->lxcpath = optarg; break;
+		case 'P':
+			remove_trailing_slashes(optarg);
+			ret = lxc_arguments_lxcpath_add(args, optarg);
+			if (ret < 0)
+				return ret;
+			break;
 		case OPT_USAGE: print_usage(args->options, args);
+		case OPT_VERSION: print_version();
 		case '?':	print_help(args, 1);
 		case 'h': 	print_help(args, 0);
 		default:
@@ -195,9 +228,16 @@ extern int lxc_arguments_parse(struct lxc_arguments *args,
 	args->argv = &argv[optind];
 	args->argc = argc - optind;
 
+	/* If no lxcpaths were given, use default */
+	if (!args->lxcpath_cnt) {
+		ret = lxc_arguments_lxcpath_add(args, lxc_global_config_value("lxc.lxcpath"));
+		if (ret < 0)
+			return ret;
+	}
+
 	/* Check the command options */
 
-	if (!args->name) {
+	if (!args->name && strcmp(args->progname, "lxc-autostart") != 0) {
 		lxc_error(args, "missing container name, use --name option");
 		return -1;
 	}
