@@ -177,7 +177,12 @@ void cgroup_disconnect(void)
 #define INIT_SCOPE "/init.scope"
 void prune_init_scope(char *cg)
 {
-	char *point = cg + strlen(cg) - strlen(INIT_SCOPE);
+	char *point;
+
+	if (!cg)
+		return;
+
+	point = cg + strlen(cg) - strlen(INIT_SCOPE);
 	if (point < cg)
 		return;
 	if (strcmp(point, INIT_SCOPE) == 0) {
