@@ -28,6 +28,7 @@
 #include <fcntl.h>
 #include <poll.h>
 #include <sys/socket.h>
+#include <inttypes.h>
 #include <sys/un.h>
 #include <sys/param.h>
 #include <malloc.h>
@@ -54,7 +55,7 @@
  * Each command is transactional, the clients send a request to
  * the server and the server answers the request with a message
  * giving the request's status (zero or a negative errno value).
- * Both the request and response may contain addtional data.
+ * Both the request and response may contain additional data.
  *
  * Each command is wrapped in a ancillary message in order to pass
  * a credential making possible to the server to check if the client
@@ -73,25 +74,59 @@
 
 lxc_log_define(lxc_commands, lxc);
 
-static int fill_sock_name(char *path, int len, const char *name,
-			  const char *inpath)
+static int fill_sock_name(char *path, int len, const char *lxcname,
+			  const char *lxcpath, const char *hashed_sock_name)
 {
-	const char *lxcpath = NULL;
+	const char *name;
+	char *tmppath;
+	size_t tmplen;
+	uint64_t hash;
 	int ret;
 
-	if (!inpath) {
+	name = lxcname;
+	if (!name)
+		name = "";
+
+	if (hashed_sock_name != NULL) {
+		ret = snprintf(path, len, "lxc/%s/command", hashed_sock_name);
+		if (ret < 0 || ret >= len) {
+			ERROR("Error writing to command sock path");
+			return -1;
+		}
+		return 0;
+	}
+
+	if (!lxcpath) {
 		lxcpath = lxc_global_config_value("lxc.lxcpath");
 		if (!lxcpath) {
 			ERROR("Out of memory getting lxcpath");
 			return -1;
 		}
 	}
-	ret = snprintf(path, len, "%s/%s/command", lxcpath ? lxcpath : inpath, name);
 
-	if (ret < 0 || ret >= len) {
-		ERROR("Name too long");
+	ret = snprintf(path, len, "%s/%s/command", lxcpath, name);
+	if (ret < 0) {
+		ERROR("Error writing to command sock path");
 		return -1;
 	}
+	if (ret < len)
+		return 0;
+
+	/* ret >= len; lxcpath or name is too long.  hash both */
+	tmplen = strlen(name) + strlen(lxcpath) + 2;
+	tmppath = alloca(tmplen);
+	ret = snprintf(tmppath, tmplen, "%s/%s", lxcpath, name);
+	if (ret < 0 || ret >= tmplen) {
+		ERROR("memory error");
+		return -1;
+	}
+	hash = fnv_64a_buf(tmppath, ret, FNV1A_64_INIT);
+	ret = snprintf(path, len, "lxc/%016" PRIx64 "/command", hash);
+	if (ret < 0 || ret >= len) {
+		ERROR("Command socket name too long");
+		return -1;
+	}
+
 	return 0;
 }
 
@@ -105,6 +140,8 @@ static const char *lxc_cmd_str(lxc_cmd_t cmd)
 		[LXC_CMD_GET_CLONE_FLAGS] = "get_clone_flags",
 		[LXC_CMD_GET_CGROUP]      = "get_cgroup",
 		[LXC_CMD_GET_CONFIG_ITEM] = "get_config_item",
+		[LXC_CMD_GET_NAME]        = "get_name",
+		[LXC_CMD_GET_LXCPATH]     = "get_lxcpath",
 	};
 
 	if (cmd >= LXC_CMD_MAX)
@@ -136,8 +173,8 @@ static int lxc_cmd_rsp_recv(int sock, struct lxc_cmd_rr *cmd)
 
 	ret = lxc_abstract_unix_recv_fd(sock, &rspfd, rsp, sizeof(*rsp));
 	if (ret < 0) {
-		ERROR("command %s failed to receive response",
-		      lxc_cmd_str(cmd->req.cmd));
+		WARN("Command %s failed to receive response: %s.",
+		     lxc_cmd_str(cmd->req.cmd), strerror(errno));
 		return -1;
 	}
 
@@ -152,7 +189,7 @@ static int lxc_cmd_rsp_recv(int sock, struct lxc_cmd_rr *cmd)
 
 		rspdata = malloc(sizeof(*rspdata));
 		if (!rspdata) {
-			ERROR("command %s couldn't allocate response buffer",
+			ERROR("Command %s couldn't allocate response buffer.",
 			      lxc_cmd_str(cmd->req.cmd));
 			return -1;
 		}
@@ -161,10 +198,13 @@ static int lxc_cmd_rsp_recv(int sock, struct lxc_cmd_rr *cmd)
 		rsp->data = rspdata;
 	}
 
-	if (rsp->datalen == 0)
+	if (rsp->datalen == 0) {
+		DEBUG("command %s response data length is 0",
+		      lxc_cmd_str(cmd->req.cmd));
 		return ret;
+	}
 	if (rsp->datalen > LXC_CMD_DATA_MAX) {
-		ERROR("command %s response data %d too long",
+		ERROR("Command %s response data %d too long.",
 		      lxc_cmd_str(cmd->req.cmd), rsp->datalen);
 		errno = EFBIG;
 		return -1;
@@ -172,14 +212,14 @@ static int lxc_cmd_rsp_recv(int sock, struct lxc_cmd_rr *cmd)
 
 	rsp->data = malloc(rsp->datalen);
 	if (!rsp->data) {
-		ERROR("command %s unable to allocate response buffer",
+		ERROR("Command %s was unable to allocate response buffer.",
 		      lxc_cmd_str(cmd->req.cmd));
 		return -1;
 	}
 	ret = recv(sock, rsp->data, rsp->datalen, 0);
 	if (ret != rsp->datalen) {
-		ERROR("command %s failed to receive response data",
-		      lxc_cmd_str(cmd->req.cmd));
+		ERROR("Command %s failed to receive response data: %s.",
+		      lxc_cmd_str(cmd->req.cmd), strerror(errno));
 		if (ret >= 0)
 			ret = -1;
 	}
@@ -201,7 +241,7 @@ static int lxc_cmd_rsp_send(int fd, struct lxc_cmd_rsp *rsp)
 
 	ret = send(fd, rsp, sizeof(*rsp), 0);
 	if (ret != sizeof(*rsp)) {
-		ERROR("failed to send command response %d %s", ret,
+		ERROR("Failed to send command response %d: %s.", ret,
 		      strerror(errno));
 		return -1;
 	}
@@ -209,8 +249,8 @@ static int lxc_cmd_rsp_send(int fd, struct lxc_cmd_rsp *rsp)
 	if (rsp->datalen > 0) {
 		ret = send(fd, rsp->data, rsp->datalen, 0);
 		if (ret != rsp->datalen) {
-			WARN("failed to send command response data %d %s", ret,
-			      strerror(errno));
+			WARN("Failed to send command response data %d: %s.",
+			     ret, strerror(errno));
 			return -1;
 		}
 	}
@@ -237,18 +277,23 @@ static int lxc_cmd_rsp_send(int fd, struct lxc_cmd_rsp *rsp)
  * returned in the cmd response structure.
  */
 static int lxc_cmd(const char *name, struct lxc_cmd_rr *cmd, int *stopped,
-		   const char *lxcpath)
+		   const char *lxcpath, const char *hashed_sock_name)
 {
 	int sock, ret = -1;
 	char path[sizeof(((struct sockaddr_un *)0)->sun_path)] = { 0 };
 	char *offset = &path[1];
-	int len;
+	size_t len;
 	int stay_connected = cmd->req.cmd == LXC_CMD_CONSOLE;
 
 	*stopped = 0;
 
-	len = sizeof(path)-1;
-	if (fill_sock_name(offset, len, name, lxcpath))
+	/* -2 here because this is an abstract unix socket so it needs a
+	 * leading \0, and we null terminate, so it needs a trailing \0.
+	 * Although null termination isn't required by the API, we do it anyway
+	 * because we print the sockname out sometimes.
+	 */
+	len = sizeof(path)-2;
+	if (fill_sock_name(offset, len, name, lxcpath, hashed_sock_name))
 		return -1;
 
 	sock = lxc_abstract_unix_connect(path);
@@ -256,7 +301,7 @@ static int lxc_cmd(const char *name, struct lxc_cmd_rr *cmd, int *stopped,
 		if (errno == ECONNREFUSED)
 			*stopped = 1;
 		else
-			SYSERROR("command %s failed to connect to '@%s'",
+			SYSERROR("Command %s failed to connect to \"@%s\".",
 				 lxc_cmd_str(cmd->req.cmd), offset);
 		return -1;
 	}
@@ -265,7 +310,7 @@ static int lxc_cmd(const char *name, struct lxc_cmd_rr *cmd, int *stopped,
 	if (ret != sizeof(cmd->req)) {
 		if (errno == EPIPE)
 			goto epipe;
-		SYSERROR("command %s failed to send req to '@%s' %d",
+		SYSERROR("Command %s failed to send req to \"@%s\" %d.",
 			 lxc_cmd_str(cmd->req.cmd), offset, ret);
 		if (ret >=0)
 			ret = -1;
@@ -277,7 +322,7 @@ static int lxc_cmd(const char *name, struct lxc_cmd_rr *cmd, int *stopped,
 		if (ret != cmd->req.datalen) {
 			if (errno == EPIPE)
 				goto epipe;
-			SYSERROR("command %s failed to send request data to '@%s' %d",
+			SYSERROR("Command %s failed to send request data to \"@%s\" %d.",
 				 lxc_cmd_str(cmd->req.cmd), offset, ret);
 			if (ret >=0)
 				ret = -1;
@@ -307,7 +352,7 @@ int lxc_try_cmd(const char *name, const char *lxcpath)
 		.req = { .cmd = LXC_CMD_GET_INIT_PID },
 	};
 
-	ret = lxc_cmd(name, &cmd, &stopped, lxcpath);
+	ret = lxc_cmd(name, &cmd, &stopped, lxcpath, NULL);
 
 	if (stopped)
 		return 0;
@@ -347,7 +392,7 @@ pid_t lxc_cmd_get_init_pid(const char *name, const char *lxcpath)
 		.req = { .cmd = LXC_CMD_GET_INIT_PID },
 	};
 
-	ret = lxc_cmd(name, &cmd, &stopped, lxcpath);
+	ret = lxc_cmd(name, &cmd, &stopped, lxcpath, NULL);
 	if (ret < 0)
 		return ret;
 
@@ -377,7 +422,7 @@ int lxc_cmd_get_clone_flags(const char *name, const char *lxcpath)
 		.req = { .cmd = LXC_CMD_GET_CLONE_FLAGS },
 	};
 
-	ret = lxc_cmd(name, &cmd, &stopped, lxcpath);
+	ret = lxc_cmd(name, &cmd, &stopped, lxcpath, NULL);
 	if (ret < 0)
 		return ret;
 
@@ -416,19 +461,18 @@ char *lxc_cmd_get_cgroup_path(const char *name, const char *lxcpath,
 		},
 	};
 
-	ret = lxc_cmd(name, &cmd, &stopped, lxcpath);
+	ret = lxc_cmd(name, &cmd, &stopped, lxcpath, NULL);
 	if (ret < 0)
 		return NULL;
 
 	if (!ret) {
-		WARN("'%s' has stopped before sending its state", name);
+		WARN("Container \"%s\" has stopped before sending its state.", name);
 		return NULL;
 	}
 
 	if (cmd.rsp.ret < 0 || cmd.rsp.datalen < 0) {
-		ERROR("command %s failed for '%s': %s",
-		      lxc_cmd_str(cmd.req.cmd), name,
-		      strerror(-cmd.rsp.ret));
+		ERROR("Command %s failed for container \"%s\": %s.",
+		      lxc_cmd_str(cmd.req.cmd), name, strerror(-cmd.rsp.ret));
 		return NULL;
 	}
 
@@ -475,7 +519,7 @@ char *lxc_cmd_get_config_item(const char *name, const char *item,
 		       },
 	};
 
-	ret = lxc_cmd(name, &cmd, &stopped, lxcpath);
+	ret = lxc_cmd(name, &cmd, &stopped, lxcpath, NULL);
 	if (ret < 0)
 		return NULL;
 
@@ -526,7 +570,7 @@ lxc_state_t lxc_cmd_get_state(const char *name, const char *lxcpath)
 		.req = { .cmd = LXC_CMD_GET_STATE }
 	};
 
-	ret = lxc_cmd(name, &cmd, &stopped, lxcpath);
+	ret = lxc_cmd(name, &cmd, &stopped, lxcpath, NULL);
 	if (ret < 0 && stopped)
 		return STOPPED;
 
@@ -534,11 +578,11 @@ lxc_state_t lxc_cmd_get_state(const char *name, const char *lxcpath)
 		return -1;
 
 	if (!ret) {
-		WARN("'%s' has stopped before sending its state", name);
+		WARN("Container \"%s\" has stopped before sending its state.", name);
 		return -1;
 	}
 
-	DEBUG("'%s' is in '%s' state", name,
+	DEBUG("Container \"%s\" is in \"%s\" state.", name,
 	      lxc_state2str(PTR_TO_INT(cmd.rsp.data)));
 	return PTR_TO_INT(cmd.rsp.data);
 }
@@ -567,10 +611,10 @@ int lxc_cmd_stop(const char *name, const char *lxcpath)
 		.req = { .cmd = LXC_CMD_STOP },
 	};
 
-	ret = lxc_cmd(name, &cmd, &stopped, lxcpath);
+	ret = lxc_cmd(name, &cmd, &stopped, lxcpath, NULL);
 	if (ret < 0) {
 		if (stopped) {
-			INFO("'%s' is already stopped", name);
+			INFO("Container \"%s\" is already stopped.", name);
 			return 0;
 		}
 		return -1;
@@ -580,11 +624,12 @@ int lxc_cmd_stop(const char *name, const char *lxcpath)
 	 * closed
 	 */
 	if (ret > 0) {
-		ERROR("failed to stop '%s': %s", name, strerror(-cmd.rsp.ret));
+		ERROR("Failed to stop container \"%s\": %s.", name,
+		      strerror(-cmd.rsp.ret));
 		return -1;
 	}
 
-	INFO("'%s' has stopped", name);
+	INFO("Container \"%s\" has stopped.", name);
 	return 0;
 }
 
@@ -606,7 +651,7 @@ static int lxc_cmd_stop_callback(int fd, struct lxc_cmd_req *req,
 		 */
 		if (cgroup_unfreeze(handler))
 			return 0;
-		ERROR("Failed to unfreeze %s:%s", handler->lxcpath, handler->name);
+		ERROR("Failed to unfreeze container \"%s\".", handler->name);
 		rsp.ret = -1;
 	}
 
@@ -628,7 +673,7 @@ int lxc_cmd_console_winch(const char *name, const char *lxcpath)
 		.req = { .cmd = LXC_CMD_CONSOLE_WINCH },
 	};
 
-	ret = lxc_cmd(name, &cmd, &stopped, lxcpath);
+	ret = lxc_cmd(name, &cmd, &stopped, lxcpath, NULL);
 	if (ret < 0)
 		return ret;
 
@@ -663,32 +708,32 @@ int lxc_cmd_console(const char *name, int *ttynum, int *fd, const char *lxcpath)
 		.req = { .cmd = LXC_CMD_CONSOLE, .data = INT_TO_PTR(*ttynum) },
 	};
 
-	ret = lxc_cmd(name, &cmd, &stopped, lxcpath);
+	ret = lxc_cmd(name, &cmd, &stopped, lxcpath, NULL);
 	if (ret < 0)
 		return ret;
 
 	if (cmd.rsp.ret < 0) {
-		ERROR("console access denied: %s", strerror(-cmd.rsp.ret));
+		ERROR("Console access denied: %s.", strerror(-cmd.rsp.ret));
 		ret = -1;
 		goto out;
 	}
 
 	if (ret == 0) {
-		ERROR("console %d invalid,busy or all consoles busy", *ttynum);
+		ERROR("Console %d invalid, busy or all consoles busy.", *ttynum);
 		ret = -1;
 		goto out;
 	}
 
 	rspdata = cmd.rsp.data;
 	if (rspdata->masterfd < 0) {
-		ERROR("unable to allocate fd for tty %d", rspdata->ttynum);
+		ERROR("Unable to allocate fd for tty %d.", rspdata->ttynum);
 		goto out;
 	}
 
 	ret = cmd.rsp.ret;	/* sock fd */
 	*fd = rspdata->masterfd;
 	*ttynum = rspdata->ttynum;
-	INFO("tty %d allocated fd %d sock %d", rspdata->ttynum, *fd, ret);
+	INFO("tty %d allocated fd %d sock %d.", rspdata->ttynum, *fd, ret);
 out:
 	free(cmd.rsp.data);
 	return ret;
@@ -708,7 +753,7 @@ static int lxc_cmd_console_callback(int fd, struct lxc_cmd_req *req,
 	memset(&rsp, 0, sizeof(rsp));
 	rsp.data = INT_TO_PTR(ttynum);
 	if (lxc_abstract_unix_send_fd(fd, masterfd, &rsp, sizeof(rsp)) < 0) {
-		ERROR("failed to send tty to client");
+		ERROR("Failed to send tty to client.");
 		lxc_console_free(handler->conf, fd);
 		goto out_close;
 	}
@@ -722,7 +767,81 @@ out_close:
 	return 1;
 }
 
+/*
+ * lxc_cmd_get_name: Returns the name of the container
+ *
+ * @hashed_sock_name: hashed socket name
+ *
+ * Returns the name on success, NULL on failure.
+ */
+char *lxc_cmd_get_name(const char *hashed_sock_name)
+{
+	int ret, stopped;
+	struct lxc_cmd_rr cmd = {
+		.req = { .cmd = LXC_CMD_GET_NAME},
+	};
 
+	ret = lxc_cmd(NULL, &cmd, &stopped, NULL, hashed_sock_name);
+	if (ret < 0) {
+		return NULL;
+	}
+
+	if (cmd.rsp.ret == 0)
+		return cmd.rsp.data;
+	return NULL;
+}
+
+static int lxc_cmd_get_name_callback(int fd, struct lxc_cmd_req *req,
+					    struct lxc_handler *handler)
+{
+	struct lxc_cmd_rsp rsp;
+
+	memset(&rsp, 0, sizeof(rsp));
+
+	rsp.data = handler->name;
+	rsp.datalen = strlen(handler->name) + 1;
+	rsp.ret = 0;
+
+	return lxc_cmd_rsp_send(fd, &rsp);
+}
+
+/*
+ * lxc_cmd_get_lxcpath: Returns the lxcpath of the container
+ *
+ * @hashed_sock_name: hashed socket name
+ *
+ * Returns the lxcpath on success, NULL on failure.
+ */
+char *lxc_cmd_get_lxcpath(const char *hashed_sock_name)
+{
+	int ret, stopped;
+	struct lxc_cmd_rr cmd = {
+		.req = { .cmd = LXC_CMD_GET_LXCPATH},
+	};
+
+	ret = lxc_cmd(NULL, &cmd, &stopped, NULL, hashed_sock_name);
+	if (ret < 0) {
+		return NULL;
+	}
+
+	if (cmd.rsp.ret == 0)
+		return cmd.rsp.data;
+	return NULL;
+}
+
+static int lxc_cmd_get_lxcpath_callback(int fd, struct lxc_cmd_req *req,
+					struct lxc_handler *handler)
+{
+	struct lxc_cmd_rsp rsp;
+
+	memset(&rsp, 0, sizeof(rsp));
+
+	rsp.data = (char *)handler->lxcpath;
+	rsp.datalen = strlen(handler->lxcpath) + 1;
+	rsp.ret = 0;
+
+	return lxc_cmd_rsp_send(fd, &rsp);
+}
 
 static int lxc_cmd_process(int fd, struct lxc_cmd_req *req,
 			   struct lxc_handler *handler)
@@ -738,10 +857,12 @@ static int lxc_cmd_process(int fd, struct lxc_cmd_req *req,
 		[LXC_CMD_GET_CLONE_FLAGS] = lxc_cmd_get_clone_flags_callback,
 		[LXC_CMD_GET_CGROUP]      = lxc_cmd_get_cgroup_callback,
 		[LXC_CMD_GET_CONFIG_ITEM] = lxc_cmd_get_config_item_callback,
+		[LXC_CMD_GET_NAME]        = lxc_cmd_get_name_callback,
+		[LXC_CMD_GET_LXCPATH]     = lxc_cmd_get_lxcpath_callback,
 	};
 
 	if (req->cmd >= LXC_CMD_MAX) {
-		ERROR("bad cmd %d received", req->cmd);
+		ERROR("Undefined command id %d received.", req->cmd);
 		return -1;
 	}
 	return cb[req->cmd](fd, req, handler);
@@ -772,23 +893,23 @@ static int lxc_cmd_handler(int fd, uint32_t events, void *data,
 	}
 
 	if (ret < 0) {
-		SYSERROR("failed to receive data on command socket");
+		SYSERROR("Failed to receive data on command socket.");
 		goto out_close;
 	}
 
 	if (!ret) {
-		DEBUG("peer has disconnected");
+		DEBUG("Peer has disconnected.");
 		goto out_close;
 	}
 
 	if (ret != sizeof(req)) {
-		WARN("partial request, ignored");
+		WARN("Failed to receive full command request. Ignoring request.");
 		ret = -1;
 		goto out_close;
 	}
 
 	if (req.datalen > LXC_CMD_DATA_MAX) {
-		ERROR("cmd data length %d too large", req.datalen);
+		ERROR("Received command data length %d is too large.", req.datalen);
 		ret = -1;
 		goto out_close;
 	}
@@ -799,7 +920,7 @@ static int lxc_cmd_handler(int fd, uint32_t events, void *data,
 		reqdata = alloca(req.datalen);
 		ret = recv(fd, reqdata, req.datalen, 0);
 		if (ret != req.datalen) {
-			WARN("partial request, ignored");
+			WARN("Failed to receive full command request. Ignoring request.");
 			ret = -1;
 			goto out_close;
 		}
@@ -827,24 +948,24 @@ static int lxc_cmd_accept(int fd, uint32_t events, void *data,
 
 	connection = accept(fd, NULL, 0);
 	if (connection < 0) {
-		SYSERROR("failed to accept connection");
+		SYSERROR("Failed to accept connection to run command.");
 		return -1;
 	}
 
 	if (fcntl(connection, F_SETFD, FD_CLOEXEC)) {
-		SYSERROR("failed to set close-on-exec on incoming connection");
+		SYSERROR("Failed to set close-on-exec on incoming command connection.");
 		goto out_close;
 	}
 
 	if (setsockopt(connection, SOL_SOCKET,
 		       SO_PASSCRED, &opt, sizeof(opt))) {
-		SYSERROR("failed to enable credential on socket");
+		SYSERROR("Failed to enable necessary credentials on command socket.");
 		goto out_close;
 	}
 
 	ret = lxc_mainloop_add_handler(descr, connection, lxc_cmd_handler, data);
 	if (ret) {
-		ERROR("failed to add handler");
+		ERROR("Failed to add command handler.");
 		goto out_close;
 	}
 
@@ -864,23 +985,26 @@ int lxc_cmd_init(const char *name, struct lxc_handler *handler,
 	char *offset = &path[1];
 	int len;
 
-	len = sizeof(path)-1;
-	if (fill_sock_name(offset, len, name, lxcpath))
+	/* -2 here because this is an abstract unix socket so it needs a
+	 * leading \0, and we null terminate, so it needs a trailing \0.
+	 * Although null termination isn't required by the API, we do it anyway
+	 * because we print the sockname out sometimes.
+	 */
+	len = sizeof(path) - 2;
+	if (fill_sock_name(offset, len, name, lxcpath, NULL))
 		return -1;
 
 	fd = lxc_abstract_unix_open(path, SOCK_STREAM, 0);
 	if (fd < 0) {
-		ERROR("failed (%d) to create the command service point %s", errno, offset);
-		if (errno == EADDRINUSE) {
-			ERROR("##");
-			ERROR("# The container appears to be already running!");
-			ERROR("##");
-		}
+		ERROR("Failed to create the command service point %s: %s.",
+		      offset, strerror(errno));
+		if (errno == EADDRINUSE)
+			ERROR("Container \"%s\" appears to be already running!", name);
 		return -1;
 	}
 
 	if (fcntl(fd, F_SETFD, FD_CLOEXEC)) {
-		SYSERROR("failed to set sigfd to close-on-exec");
+		SYSERROR("Failed to set FD_CLOEXEC on signal file descriptor.");
 		close(fd);
 		return -1;
 	}
@@ -897,7 +1021,7 @@ int lxc_cmd_mainloop_add(const char *name,
 
 	ret = lxc_mainloop_add_handler(descr, fd, lxc_cmd_accept, handler);
 	if (ret) {
-		ERROR("failed to add handler for command socket");
+		ERROR("Failed to add handler for command socket.");
 		close(fd);
 	}
 
