@@ -20,8 +20,8 @@
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
-#ifndef __lxc_error_h
-#define __lxc_error_h
+#ifndef __LXC_ERROR_H
+#define __LXC_ERROR_H
 
 extern int  lxc_error_set_and_log(int pid, int status);
 
