@@ -50,9 +50,13 @@ static int filter_count = 0;
 
 static int my_parser(struct lxc_arguments* args, int c, char* arg)
 {
+	char **newk;
 	switch (c) {
 	case 'c':
-		key = realloc(key, keys+1 * sizeof(key[0]));
+		newk = realloc(key, (keys + 1) * sizeof(key[0]));
+		if (!newk)
+			return -1;
+		key = newk;
 		key[keys] = arg;
 		keys++;
 		break;
@@ -83,7 +87,7 @@ static struct lxc_arguments my_args = {
 lxc-info display some information about a container with the identifier NAME\n\
 \n\
 Options :\n\
-  -n, --name=NAME       NAME for name of the container\n\
+  -n, --name=NAME       NAME of the container\n\
   -c, --config=KEY      show configuration variable KEY from running container\n\
   -i, --ips             shows the IP addresses\n\
   -p, --pid             shows the process id of the init container\n\
@@ -165,6 +169,7 @@ static void print_net_stats(struct lxc_container *c)
 		if (!ifname)
 			return;
 		printf("%-15s %s\n", "Link:", ifname);
+		fflush(stdout);
 
 		/* XXX: tx and rx are reversed from the host vs container
 		 * perspective, print them from the container perspective
@@ -175,6 +180,7 @@ static void print_net_stats(struct lxc_container *c)
 			str_chomp(buf);
 			rx_bytes = str_size_humanize(buf, sizeof(buf));
 			printf("%-15s %s\n", " TX bytes:", buf);
+			fflush(stdout);
 		}
 
 		snprintf(path, sizeof(path), "/sys/class/net/%s/statistics/tx_bytes", ifname);
@@ -183,11 +189,13 @@ static void print_net_stats(struct lxc_container *c)
 			str_chomp(buf);
 			tx_bytes = str_size_humanize(buf, sizeof(buf));
 			printf("%-15s %s\n", " RX bytes:", buf);
+			fflush(stdout);
 		}
 
 		sprintf(buf, "%llu", rx_bytes + tx_bytes);
 		str_size_humanize(buf, sizeof(buf));
 		printf("%-15s %s\n", " Total bytes:", buf);
+		fflush(stdout);
 		free(ifname);
 	}
 }
@@ -195,7 +203,7 @@ static void print_net_stats(struct lxc_container *c)
 static void print_stats(struct lxc_container *c)
 {
 	int i, ret;
-	char buf[256];
+	char buf[4096];
 
 	ret = c->get_cgroup_item(c, "cpuacct.usage", buf, sizeof(buf));
 	if (ret > 0 && ret < sizeof(buf)) {
@@ -206,6 +214,7 @@ static void print_stats(struct lxc_container *c)
 		} else {
 			printf("%-15s %s\n", "CPU use:", buf);
 		}
+		fflush(stdout);
 	}
 
 	ret = c->get_cgroup_item(c, "blkio.throttle.io_service_bytes", buf, sizeof(buf));
@@ -225,6 +234,7 @@ static void print_stats(struct lxc_container *c)
 			str_size_humanize(buf, sizeof(buf));
 			printf("%-15s %s\n", "BlkIO use:", buf);
 		}
+		fflush(stdout);
 	}
 
 	static const struct {
@@ -242,6 +252,7 @@ static void print_stats(struct lxc_container *c)
 			str_chomp(buf);
 			str_size_humanize(buf, sizeof(buf));
 			printf("%-15s %s\n", lxstat[i].name, buf);
+			fflush(stdout);
 		}
 	}
 }
@@ -256,6 +267,7 @@ static void print_info_msg_int(const char *key, int value)
 		else
 			printf("%-15s %d\n", key, value);
 	}
+	fflush(stdout);
 }
 
 static void print_info_msg_str(const char *key, const char *value)
@@ -268,6 +280,7 @@ static void print_info_msg_str(const char *key, const char *value)
 		else
 			printf("%-15s %s\n", key, value);
 	}
+	fflush(stdout);
 }
 
 static int print_info(const char *name, const char *lxcpath)
@@ -355,6 +368,7 @@ static int print_info(const char *name, const char *lxcpath)
 		} else {
 			fprintf(stderr, "%s invalid\n", key[i]);
 		}
+		fflush(stdout);
 	}
 
 	lxc_container_put(c);
