@@ -38,6 +38,7 @@
 #include <sys/wait.h>
 #include <netinet/in.h>
 #include <net/if.h>
+#include <poll.h>
 
 #include "error.h"
 #include "af_unix.h"
@@ -106,8 +107,10 @@ static void lxc_monitor_fifo_send(struct lxc_msg *msg, const char *lxcpath)
 		return;
 	}
 
-	if (fcntl(fd, F_SETFL, O_WRONLY) < 0)
+	if (fcntl(fd, F_SETFL, O_WRONLY) < 0) {
+		close(fd);
 		return;
+	}
 
 	ret = write(fd, msg, sizeof(*msg));
 	if (ret != sizeof(*msg)) {
@@ -129,6 +132,16 @@ void lxc_monitor_send_state(const char *name, lxc_state_t state, const char *lxc
 	lxc_monitor_fifo_send(&msg, lxcpath);
 }
 
+void lxc_monitor_send_exit_code(const char *name, int exit_code, const char *lxcpath)
+{
+	struct lxc_msg msg = { .type = lxc_msg_exit_code,
+			       .value = exit_code };
+	strncpy(msg.name, name, sizeof(msg.name));
+	msg.name[sizeof(msg.name) - 1] = 0;
+
+	lxc_monitor_fifo_send(&msg, lxcpath);
+}
+
 
 /* routines used by monitor subscribers (lxc-monitor) */
 int lxc_monitor_close(int fd)
@@ -140,7 +153,7 @@ int lxc_monitor_sock_name(const char *lxcpath, struct sockaddr_un *addr) {
 	size_t len;
 	int ret;
 	char *sockname = &addr->sun_path[1];
-	char path[PATH_MAX+18];
+	char *path;
 	uint64_t hash;
 
 	/* addr.sun_path is only 108 bytes, so we hash the full name and
@@ -148,18 +161,20 @@ int lxc_monitor_sock_name(const char *lxcpath, struct sockaddr_un *addr) {
 	 */
 	memset(addr, 0, sizeof(*addr));
 	addr->sun_family = AF_UNIX;
-	len = sizeof(addr->sun_path) - 1;
-	ret = snprintf(path, sizeof(path), "lxc/%s/monitor-sock", lxcpath);
-	if (ret < 0 || ret >= sizeof(path)) {
-		ERROR("lxcpath %s too long for monitor unix socket", lxcpath);
+	len = strlen(lxcpath) + 18;
+	path = alloca(len);
+	ret = snprintf(path, len, "lxc/%s/monitor-sock", lxcpath);
+	if (ret < 0 || ret >= len) {
+		ERROR("memory error creating monitor path");
 		return -1;
 	}
 
+	len = sizeof(addr->sun_path) - 1;
 	hash = fnv_64a_buf(path, ret, FNV1A_64_INIT);
 	ret = snprintf(sockname, len, "lxc/%016" PRIx64 "/%s", hash, lxcpath);
 	if (ret < 0)
 		return -1;
-	sockname[sizeof(addr->sun_path)-2] = '\0';
+	sockname[sizeof(addr->sun_path)-3] = '\0';
 	INFO("using monitor sock name %s", sockname);
 	return 0;
 }
@@ -167,7 +182,7 @@ int lxc_monitor_sock_name(const char *lxcpath, struct sockaddr_un *addr) {
 int lxc_monitor_open(const char *lxcpath)
 {
 	struct sockaddr_un addr;
-	int fd,ret;
+	int fd,ret = 0;
 	int retry,backoff_ms[] = {10, 50, 100};
 	size_t len;
 
@@ -205,19 +220,13 @@ err1:
 	return ret;
 }
 
-int lxc_monitor_read_fdset(fd_set *rfds, int nfds, struct lxc_msg *msg,
+int lxc_monitor_read_fdset(struct pollfd *fds, nfds_t nfds, struct lxc_msg *msg,
 			   int timeout)
 {
-	struct timeval tval,*tv = NULL;
-	int ret,i;
+	long i;
+	int ret;
 
-	if (timeout != -1) {
-		tv = &tval;
-		tv->tv_sec = timeout;
-		tv->tv_usec = 0;
-	}
-
-	ret = select(nfds, rfds, NULL, NULL, tv);
+	ret = poll(fds, nfds, timeout * 1000);
 	if (ret == -1)
 		return -1;
 	else if (ret == 0)
@@ -227,8 +236,9 @@ int lxc_monitor_read_fdset(fd_set *rfds, int nfds, struct lxc_msg *msg,
 	 * for when this routine is called again
 	 */
 	for (i = 0; i < nfds; i++) {
-		if (FD_ISSET(i, rfds)) {
-			ret = recv(i, msg, sizeof(*msg), 0);
+		if (fds[i].revents != 0) {
+			fds[i].revents = 0;
+			ret = recv(fds[i].fd, msg, sizeof(*msg), 0);
 			if (ret <= 0) {
 				SYSERROR("client failed to recv (monitord died?) %s",
 					 strerror(errno));
@@ -243,12 +253,13 @@ int lxc_monitor_read_fdset(fd_set *rfds, int nfds, struct lxc_msg *msg,
 
 int lxc_monitor_read_timeout(int fd, struct lxc_msg *msg, int timeout)
 {
-	fd_set rfds;
+	struct pollfd fds;
 
-	FD_ZERO(&rfds);
-	FD_SET(fd, &rfds);
+	fds.fd = fd;
+	fds.events = POLLIN | POLLPRI;
+	fds.revents = 0;
 
-	return lxc_monitor_read_fdset(&rfds, fd+1, msg, timeout);
+	return lxc_monitor_read_fdset(&fds, 1, msg, timeout);
 }
 
 int lxc_monitor_read(int fd, struct lxc_msg *msg)
@@ -317,12 +328,9 @@ int lxc_monitord_spawn(const char *lxcpath)
 		SYSERROR("failed to setsid");
 		exit(EXIT_FAILURE);
 	}
-	close(0);
-	close(1);
-	close(2);
-	open("/dev/null", O_RDONLY);
-	open("/dev/null", O_RDWR);
-	open("/dev/null", O_RDWR);
+	lxc_check_inherited(NULL, pipefd[1]);
+	if (null_stdfds() < 0)
+		exit(EXIT_FAILURE);
 	close(pipefd[0]);
 	sprintf(pipefd_str, "%d", pipefd[1]);
 	execvp(args[0], args);
