@@ -20,8 +20,8 @@
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
-#ifndef __namespace_h
-#define __namespace_h
+#ifndef __LXC_NAMESPACE_H
+#define __LXC_NAMESPACE_H
 
 #include <sys/syscall.h>
 #include <sched.h>
