@@ -1,4 +1,5 @@
-#!/usr/bin/python3
+#!/usr/bin/env python3
+# -*- coding: utf-8 -*-
 #
 # api_test.py: Test/demo of the python3-lxc API
 #
@@ -91,7 +92,6 @@ print("Testing the networking")
 # A few basic checks of the current state
 assert("name" in container.get_keys("lxc.network.0"))
 assert(len(container.network) == 1)
-assert(container.network[0].hwaddr.startswith("00:16:3e"))
 
 ## Starting the container
 print("Starting the container")
@@ -181,6 +181,14 @@ assert(container.init_pid == -1)
 assert(not container.running)
 assert(container.state == "STOPPED")
 
+## Snapshotting the container
+print("Snapshotting the container")
+assert(not container.snapshot_list())
+assert(container.snapshot() == "snap0")
+assert(len(container.snapshot_list()) == 1)
+assert(container.snapshot_restore("snap0") is True)
+assert(container.snapshot_destroy("snap0") is True)
+
 ## Cloning the container
 print("Cloning the container as '%s'" % CLONE_NAME)
 clone = container.clone(CLONE_NAME)
